//
// Wood Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//

//string XFile = "misc\\teapot.x";
string XFile = "teapot.x";
int    BCLR  = 0xff202080;

// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;

// light direction (view space)
half3 lightDir <  string UIDirectional = "Light Direction"; > = {0.577, -0.577, 0.577};

// light intensity
half4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
half4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
half4 I_s = { 0.6f, 3.0f, 0.6f, 1.0f };    // specular

// material reflectivity
half4 k_a = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient
half4 k_d = { 0.7f, 0.2f, 1.0f, 1.0f };    // diffuse
half4 k_s = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular
half  n   = 64.0f;                         // power

half3 k_d_negative = { 0.35f, 0.4f, 1.0f }; // an inverse amount of the diffuse, for vs-only specular


#define DARK    0.3f
#define LIGHT   1.0f

// model dependent wood parameters
#define RINGSCALE   10.0f
#define POINTSCALE  2.0f
#define TURBULENCE  1.0f
#define VOLUME_SIZE 32

half time : TIME;

// textures



texture NoiseTexCube
< 
    string type = "CUBE"; 
    string function = "GenerateNoiseCube"; 
    int width = 128;
    int height = 128;
>;



// function used to fill the volume noise texture
half4 GenerateNoiseCube(half3 Pos : POSITION) : COLOR
{
    half4 Color;
    half3 tex1 = 0.5 * Pos * POINTSCALE;
    half3 tex2 = 0.5 * (Pos+.01) * POINTSCALE;
    half3 tex3 = 0.5 * (Pos+.02) * POINTSCALE;

//    half ringdist = (Pos.x*Pos.x + Pos.z*Pos.z);

//    half r = 4*Pos.y - ringdist + .75 + .5*(noise(tex1*14.3)-.5);
    half r = 1*(noise(tex1*14.3)-.5);
    half g = 1*(noise(tex2*18.3)-.5);
    half b = 1*(noise(tex3*14.3)-.5);

//    if(Pos.y == -1 && ringdist < 1)
//    r -= 5*ringdist;

//    if(Pos.y == 1 && ringdist < 1)
//    r += 5*ringdist;

//	if(r > )
//	r = 0;
    r = frac(sin(r*3.14159/2));
    g = frac(sin(g*3.14159/2));
    b = frac(sin(b*3.14159/2));

    Color.r = lerp(DARK, LIGHT, saturate(r));
    Color.g = lerp(DARK, LIGHT, saturate(g));
    Color.b = lerp(DARK, LIGHT, saturate(b));
    Color.rgb *= I_a * k_a + I_d * k_d;
    Color.w   = 1;

    return Color;
}

struct VSforFF_OUTPUT
{
    half4 Pos  : POSITION;
    half3 Diff : COLOR0;
    half3 Tex0 : TEXCOORD0;               
};

VSforFF_OUTPUT VSforFF(    
    half3 Pos  : POSITION,
    half3 Norm : NORMAL)
{
    VSforFF_OUTPUT Out = (VSforFF_OUTPUT)0;

    half3 L = -lightDir;
    half3 P = mul(half4(Pos, 1), (half4x3)WorldView);    // position (view space)
    half3 N = normalize(mul(Norm, (half3x3)WorldView));   // normal (view space)
    half3 R = normalize(2 * dot(N, L) * N - L);            // reflection vector (view space)
    half3 V = -normalize(P);                               // view direction (view space)

    Out.Pos  = mul(half4(P, 1), Projection);             // position (projected)
    Out.Diff = 0.5*max(0, dot(N, L))  // diffuse + ambient
             + 2.0*I_s * (min(k_s,k_d_negative)) * pow(max(0, dot(R, V)), n/4);   // specular
    Out.Tex0 = 0.5 * sin(Pos*2) * POINTSCALE + 0.1*(sin(Pos.x)+sin(Pos.y)+sin(Pos.z)) * POINTSCALE ;
    return Out;
}



technique TAlien
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSforFF();
        PixelShader = NULL;

        DitherEnable = TRUE;
        AlphaBlendEnable = FALSE;
        CullMode         = CCW;

Lighting = false;
SpecularEnable = false;

        AlphaBlendEnable = FALSE;
        CullMode         = CCW;
        DitherEnable = TRUE;

ZEnable = true;
ZWriteEnable = true;

        // texture
        Texture[0] = (NoiseTexCube);
//        Texture[1] = (EnvironmentMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = NONE;

        MinFilter[1] = LINEAR;
        MagFilter[1] = LINEAR;
        MipFilter[1] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
//        ColorOp[0]   = ADDSIGNED2X;
        ColorOp[0]   = MODULATE2X;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

//        ColorOp[1]   = BLENDDIFFUSEALPHA;
        ColorOp[1]   = DISABLE;
        ColorArg1[1] = TEXTURE;
        ColorArg2[1] = CURRENT;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = FALSE;
        SrcBlend         = ONE;
        DestBlend        = ZERO;
//        BlendOp = SUBTRACT;
    }
}

