//
// Glow Effect
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//


half4x3 WorldView  : WORLDVIEW; 
half4x4 Projection : PROJECTION;
half3 LightDir < string UIDirectional = "Light Direction"; > = normalize(half3(0.0f, 0.0f, 1.0f));
half  MeshRadius : MESHRADIUS = 4.025f;

half4 AmbientSide = half4(0.0,0.0,0.0,0);
half4 AmbientFull = half4(0.125,0.125,0.25,0);
//half4 AmbientFull = half4(0,0,0,0);

//half3 LightPos = (half3(0.0f, 0.0f, 100.0f));
half  LightRange = 175.0f;
//half3 LightPos = (half3(50.0f, -50.0f, 75.0f));

half4x4 _world : WORLD; 
half4x4 View  : VIEW; 

string XFile = "tiny.x";   // model
string BIMG  = "lake.bmp";  // Background image
int    BCLR  = 0xff202080;  // background

// texture
texture Tex0 : TEXTURE;

struct VSTEXTURE_OUTPUT
{
    half4 Position : POSITION;
    half4 Diffuse  : COLOR;
    half2 TexCoord : TEXCOORD0;
};

// draws unskinned object with one texture and one directional light.
VSTEXTURE_OUTPUT VSTexture
    (
    half4 Position : POSITION, 
    half3 Normal   : NORMAL,
    half2 TexCoord : TEXCOORD0
    )
{
    VSTEXTURE_OUTPUT Out = (VSTEXTURE_OUTPUT)0;
  
    half3 P = mul(Position, WorldView);          // position (view space)
    half3 N = mul(Normal, (half3x3)WorldView);  // normal (view space)

half3 LightPos = -LightDir * 100;

    half3 PWorld = mul(Position, _world);          // position (world space)
    half3 LWorld = mul(LightPos, _world);          // position (world space)

    half4 PProj = mul(half4(PWorld , 1), Projection);          // position (world space)
    half4 LProj = mul(half4(LWorld , 1), Projection);          // position (world space)

   half3 LToV = PWorld - LightPos;

    Out.TexCoord = TexCoord;                            // texture coordinates
    Out.Position = mul(half4(P, 1), Projection);       // projected position
//    half dist = distance(LightPos, Position);
    half dist = length(LToV);
//    if(dist < LightRange)
    {

        half3 LDir = normalize(LToV);


        Out.Diffuse = max(0, -dot(N, LDir)) + AmbientSide; // calculate lighting from orientation 
//        Out.Diffuse *= lerp(1,0,dist*dist/LightRange/LightRange); // calculate lighting from distance
        Out.Diffuse *= 1/(0.01*dist); // calculate lighting from distance
    }
    Out.Diffuse += AmbientFull; // Add ambient lighting

    return Out;    
}

struct VSOUTLINE_OUTPUT
{
    half4 Position : POSITION;
//    half4 Diffuse  : COLOR;
};

// draws an outline around the object
VSOUTLINE_OUTPUT VSOutline
    (
    half4 Position : POSITION, 
    half3 Normal   : NORMAL
    )
{
    VSOUTLINE_OUTPUT Out = (VSOUTLINE_OUTPUT)0;
  
    half3 N = /*normalize*/(mul(Normal, (half3x3)WorldView)); // normal (view space)
    half3 P = mul(Position, WorldView) + MeshRadius/32 * N;// position (view space)
   
    Out.Position = mul(half4(P, 1), Projection);   // projected position

    return Out;    
}


technique TOutlineAndTextureWithVertexShader
{
    pass POutline
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSOutline();
        PixelShader  = NULL;
        
        // no texture
        Texture[0] = NULL;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = DISABLE;
        ColorArg1[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
//        SrcBlend = DESTCOLOR;
//        DestBlend = INVDESTCOLOR;
        SrcBlend = ZERO;
        DestBlend = ZERO;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;
    }

    pass PTexture
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSTexture();
        PixelShader  = NULL;
        
        // texture
        Texture[0] = (Tex0);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = FALSE;
        SrcBlend = ONE;
        DestBlend = ZERO;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = TRUE;
    }
}



// light intensity
half4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse

// material reflectivity
half4 k_a : MATERIALAMBIENT = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient
half4 k_d : MATERIALDIFFUSE = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
//half4 k_s : MATERIALSPECULAR= { 1.0f, 1.0f, 1.0f, 1.0f };    // specular
//int    n   : MATERIALPOWER = 32;                            // power

half g = 1.075;

technique TOutlineAndTextureNoShader
{
    pass SHADOW0
    {
        // transforms
        WorldTransform[0]   = (_world);
//        WorldTransform[0]   = (_world * matrix(g,g,g,1, g,g,g,1, g,g,g,1, 1,1,1,1));
        ViewTransform       = (View);
        ProjectionTransform = (Projection + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, 0,-2*MeshRadius/64,0,0));

        Lighting       = FALSE;
        
        // no texture
        Texture[0] = NULL;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = DISABLE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ZERO;
        DestBlend = ZERO;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;

    }
    pass SHADOW1
    {
        ProjectionTransform = (Projection + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, -1.414*MeshRadius/64,1.414*MeshRadius/64,0,0));
    }
    pass SHADOW2
    {
        ProjectionTransform = (Projection + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, 1.414*MeshRadius/64,1.414*MeshRadius/64,0,0));
    }
    pass P1
    {
        // transforms
        WorldTransform[0]   = (_world);
        ViewTransform       = (View);
        ProjectionTransform = (Projection);

        // material
        MaterialAmbient  = (half4(1,1,1,0)); 
        MaterialDiffuse  = (k_d); 
//        MaterialSpecular = (k_s); 
//        MaterialPower    = (n);
        MaterialSpecular = (0); 
        MaterialPower    = (0);
        
        // lighting
//        LightType[0]      = DIRECTIONAL;
        LightType[0]      = POINT;
        LightAmbient[0]   = (AmbientSide);
        Ambient = (AmbientFull);
        LightDiffuse[0]   = (I_d);
//        LightSpecular[0]  = (I_s); 
        LightSpecular[0]  = (0); 
//        LightDirection[0] = (LightDir);
//        LightPosition[0] = (-LightDir * 100);



//        LightPosition[0] = (fLightPos);
        LightPosition[0] = (-LightDir * 100);
//        LightRange[0]     = 100000.0f;
        LightRange[0]     = (LightRange*9999);
        LightAttenuation0[0] = (0.01);
//        LightAttenuation1[0] = (0.00015*LightRange);
        LightAttenuation1[0] = (0.0);
        LightAttenuation2[0] = (-0.0001);

        LightAttenuation0[0] = (0.0);
        LightAttenuation1[0] = (0.01);
        LightAttenuation2[0] = (0.0);

//        LightAttenuation0[0] = (0.0);
  //      LightAttenuation1[0] = (1.0);
    //    LightAttenuation2[0] = (0.0);


        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = FALSE;
        
        // texture
        Texture[0] = (Tex0);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;
        
        // texture stages
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = MODULATE;
        AlphaArg1[0] = TEXTURE;
        AlphaArg2[0] = DIFFUSE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = FALSE;
        SrcBlend = ONE;
        DestBlend = ZERO;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = TRUE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}

