//
// Metallic Flakes Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//
// NOTE:
// The metallic surface consists of 2 layers - 
// 1. a polished layer of wax on top (contributes a smooth specular reflection and an environment mapped reflection with a Fresnel term), and 
// 2. a blue metallic layer with a sprinkling of gold metallic flakes underneath
//
// newest of this

// sparkle parameters
//#define SPRINKLE    0.3
//#define SCATTER     0.3

//#define VOLUME_NOISE_SCALE  10

// scene info (EffectEdit specific controls)
string XFile = "skullocc.x";
string BIMG  = "ground2.bmp";  // Background image
int    BCLR  = 0xff404080;

// textures

texture TinyMap < string name = "Data\\Images\\Effects\\env3.bmp" ; >;

// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;
half4x4 WorldViewProj : WORLDVIEWPROJECTION;

// light direction (view space)
half3 L < string UIDirectional = "Light Direction"; > = normalize(half3(-0.397f, -0.397f, 0.827f));

// light intensity
half4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
half4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
half4 I_s = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular

// material reflectivity
//half4 k_a : MATERIALAMBIENT = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient  (metal)
//half4 k_d : MATERIALDIFFUSE = { 0.1f, 0.1f, 0.9f, 1.0f };    // diffuse  (metal)
//half4 k_s = { 0.4f, 0.3f, 0.1f, 1.0f };    // specular (metal)
half4 k_r = { 2.5f, 2.5f, 2.5f, 1.5f };    // specular (wax)


half time : TIME;









struct VS_OUTPUT
{
    half4 Position   : POSITION;
    half4 Diffuse    : COLOR0;
//    half4 Specular   : COLOR1;               
    half2 TexCoord   : TEXCOORD0;
//    half3 Reflection : TEXCOORD0;               
//    half3 NoiseCoord : TEXCOORD1;               
//    half3 Glossiness : TEXCOORD2;               
//    half3 HalfVector : TEXCOORD3;
};


// vertex shader
VS_OUTPUT VS(    
    half3 Position : POSITION,
    half3 Normal   : NORMAL, 
//    half3 Tangent  : TANGENT,
    half3 Color    : COLOR,
    half3 Binormal : BINORMAL,
    half2 TexCoord : TEXCOORD0)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    L = -L;

//    half sint = (sin(time/2)+1.1)/2;
//    Normal += lerp(sin(Normal*Normal*time*2),0,1-sint*sint*sint*sint);

    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)
    half3 P = mul(half4(Position, 1), (half4x3)WorldView);   // position (view space)
    half3 R = normalize(2 * dot(N, L) * N - L);                // reflection vector (view space)
    half3 V = -normalize(P);                                   // view direction (view space)
    half3 G = normalize(2 * dot(N, V) * N - V);                // glance vector (view space)
    half3 H = normalize(L + V);                                // half vector (view space)
    half  f = 0.5 - dot(V, N); f = 1 - 4 * f * f;              // fresnel term

    // position (projected)
//    Out.Position = mul(half4(P, 1), Projection) + half4(0,0,-.01,0);             
    Out.Position = mul(half4(P, 1), Projection);             

    // specular (wax)
    half Specular  = saturate(dot(H, N));
    Specular = (half)pow(Specular,32);
    Specular *= k_r;                                       

     // glossiness (wax)
    half4 Glossiness = f * k_r;

    // environment map coordinates
    half3 Reflection = half3(-G.x, G.y, -G.z);                   


    Out.TexCoord = half2(N.x,N.y) + half2(sin(dot(G, Normal)),sin(f));

    Out.Diffuse = 0.2;
    Out.Diffuse *= Glossiness/2;
    Out.Diffuse += min(half4(.5,1,.6,0),Specular/2 * half4(1,0.75,1,1));
    Out.Diffuse += half4(-N.y/2,0,0,0);
    Out.Diffuse.a = 1;

    return Out;


}

// technique
technique TWaxyOilyShinyMetallicTransparent
{
/*   pass ZBufferUnderPass
    {
        VertexShader = NULL;
        PixelShader = NULL;

        CullMode         = CCW;
        ZEnable         = TRUE;
        ZWriteEnable         = TRUE;

        AlphaBlendEnable = TRUE;
        ColorOp[0] = DISABLE;
        AlphaOp[0] = DISABLE;
        SrcBlend = ZERO;
        DestBlend = ONE;
    }
*/
   pass WaxyOilyShinyMetallicPassTransparent
    {
        VertexShader = compile vs_1_1 VS();
//        PixelShader  = compile ps_1_1 PS();
        PixelShader = NULL;

//        AlphaBlendEnable = TRUE;
        CullMode         = CCW;
        ZEnable         = TRUE;
        ZWriteEnable         = TRUE;
        DitherEnable = TRUE;

        // texture
//        Texture[0] = (EnvironmentMap);
        Texture[0] = (TinyMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        AlphaOp[0]   = DISABLE;
//        AlphaOp[0]   = SELECTARG1;
//        AlphaArg1[0] = DIFFUSE;
//        AlphaArg1[0] = DIFFUSE;
//        AlphaArg2[0] = CURRENT;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = FALSE;
  //     AlphaBlendEnable = TRUE;
  //      SrcBlend         = ONE;
  //      DestBlend        = ONE;
//        BlendOp = SUBTRACT;


    }
}

