//
// Force Field / Base Shield Glow Effect
//

half4x3 WorldView  : WORLDVIEW; 
half4x4 WorldViewProj  : WORLDVIEWPROJECTION; 
half3 shieldColor : CUSTOMCOLOR0 = half3(1,1,1);
texture TexE < string name = "Data\\Images\\Effects\\dot.bmp"; >;

struct VSGLOW_OUTPUT
{
    half4 Position : POSITION; // outputs
    half3 Diffuse  : COLOR;
    half2 TexCoord : TEXCOORD;
};

VSGLOW_OUTPUT VSGlow
(
	half4 Position : POSITION,  // inputs
	half3 Normal   : NORMAL,
	uniform half3 gco
)
{
    VSGLOW_OUTPUT Out = (VSGLOW_OUTPUT)0;

    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)

    Out.TexCoord = half2(N.z*0.875f, N.z*0.875f);

	Out.Diffuse = gco;

    Out.Position = mul(Position, WorldViewProj);   // projected position

    return Out;    
}

technique TGlowShield
{
    pass PGlowLayer1
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSGlow(shieldColor);
        PixelShader  = NULL;

        Texture[0] = (TexE);
        MipFilter[0] = NONE;

        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
        DestBlend = ONE;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;
        BlendOp = ADD;
     }
}
technique TGlowShieldSubtract
{
    pass PGlowLayer1
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSGlow(shieldColor);
        PixelShader  = NULL;

        Texture[0] = (TexE);
        MipFilter[0] = NONE;

        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
        DestBlend = ONE;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;
        BlendOp = REVSUBTRACT;
     }
}









struct VSGLOWOUTLINE_OUTPUT
{
    half4 Position : POSITION; // outputs
    half3 Diffuse  : COLOR;
};

VSGLOWOUTLINE_OUTPUT VSGlowOutline
(
	half4 Position : POSITION,  // inputs
	half3 Normal   : NORMAL,
	uniform half3 gco
)
{
    VSGLOWOUTLINE_OUTPUT Out = (VSGLOWOUTLINE_OUTPUT)0;

    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)

	half val = 1+N.z;
	Out.Diffuse = gco * (val*val*val);

    Out.Position = mul(Position, WorldViewProj);   // projected position

    return Out;    
}

technique TGlowOutline
{
    pass PGlowLayer1
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSGlowOutline(shieldColor);
        PixelShader  = NULL;

        Texture[0] = (TexE);
        MipFilter[0] = NONE;

        ColorOp[0]   = SELECTARG1;
        ColorArg1[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = INVDESTCOLOR;
        DestBlend = ONE;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;
        BlendOp = ADD;
     }
}