//
// Note: This effect file works with EffectEdit.
//


// scene info (EffectEdit specific controls)
string XFile = "teapot.x";
int    BCLR  = 0xff404080;

// textures
texture EnvironmentMap 
< 
    string name = "Data\\Images\\Textures\\heavymap.bmp"; 
>;

float3 diffuse : DIFFUSE;

// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;

// light direction (view space)
half3 L < string UIDirectional = "Light Direction"; > = {0.577, 0.577, -.577};


struct modVSforFF_OUTPUT
{
    half4 Position   : POSITION;
    half2 TexCoord   : TEXCOORD; // envmap
    half3 Diffuse    : COLOR0;
};

// vertex shader whose output goes to the fixed function pipeline
modVSforFF_OUTPUT modVSforFF(
    half3 Position : POSITION,
    half3 Normal   : NORMAL)
{
    modVSforFF_OUTPUT Out = (modVSforFF_OUTPUT)0;

    half3 P = mul(half4(Position, 1), (half4x3)WorldView);   // position (view space)
    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)
    half3 V = -normalize(P);                                   // view direction (view space)

    // position (projected)
    Out.Position = mul(half4(P, 1), Projection);             

    // environment/lighting map coordinates
    half viewAlign = dot(N,V);
    half brightness = max(0.1, min(0.9, viewAlign ));
    half glanceY = (2 * viewAlign * N.y - V.y);
    half skyward = lerp(0,1,(1-glanceY)/2);
    Out.TexCoord = half2(brightness, skyward);

	Out.Diffuse = diffuse;

    return Out;
}



technique tecMod
{
    pass P0
    {
        VertexShader = compile vs_1_1 modVSforFF();
        PixelShader = NULL;

        // texture
        Texture[0] = (EnvironmentMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        MinFilter[1] = LINEAR;
        MagFilter[1] = LINEAR;
        MipFilter[1] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE2X;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = FALSE;
        SrcBlend         = ONE;
        DestBlend        = ZERO;
        CullMode         = CCW;
        DitherEnable = TRUE;
        
        SpecularEnable = false;
    }
}




struct overVSforFF_OUTPUT
{
    half4 Position   : POSITION;
    half2 TexCoord   : TEXCOORD; // envmap
};

// vertex shader whose output goes to the fixed function pipeline
overVSforFF_OUTPUT overVSforFF(
    half3 Position : POSITION,
    half3 Normal   : NORMAL)
{
    overVSforFF_OUTPUT Out = (overVSforFF_OUTPUT)0;

    half3 P = mul(half4(Position, 1), (half4x3)WorldView);   // position (view space)
    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)
    half3 V = -normalize(P);                                   // view direction (view space)

    // position (projected)
    Out.Position = mul(half4(P, 1), Projection);             

    // environment/lighting map coordinates
    half viewAlign = dot(N,V);
    half brightness = max(0.1, min(0.9, viewAlign ));
    half glanceY = (2 * viewAlign * N.y - V.y);
    half skyward = lerp(0,1,(1-glanceY)/2);
    Out.TexCoord = half2(brightness, skyward);

    return Out;
}

technique tecOver
{
    pass P0
    {
        VertexShader = compile vs_1_1 overVSforFF();
        PixelShader = NULL;

        // texture
        Texture[0] = (EnvironmentMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        MinFilter[1] = LINEAR;
        MagFilter[1] = LINEAR;
        MipFilter[1] = LINEAR;

        // set up texture stage states for single texture 
        ColorOp[0]   = SELECTARG1;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = FALSE;
        SrcBlend         = ONE;
        DestBlend        = ZERO;
        CullMode         = CCW;
        DitherEnable = TRUE;
        
        SpecularEnable = false;
    }
}

