//
// Glow Effect
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//

string XFile = "tiny.x";   // model
string BIMG  = "lake.bmp";  // Background image
int    BCLR  = 0xff202080;  // background

// texture
texture Tex0 < string name = "tiny_skin.bmp"; >;

// transforms
float4x3 WorldView  : WORLDVIEW; 
float4x4 Projection : PROJECTION;

float time : TIME;

float GetOscillatingValue(float4 min_max_speed_phase)
{
  return min_max_speed_phase.x + (min_max_speed_phase.y - min_max_speed_phase.x)/2 * (1 + sin(min_max_speed_phase.w + time*min_max_speed_phase.z));
}


// light direction (view space)
float3 LightDir < string UIDirectional = "Light Direction"; > = normalize(float3(0.0f, 0.0f, 1.0f));

// glow parameters
float4 GlowColor     = float4(0.8f, 0.4f, 0.0f, 1.0f);
//float4 GlowAmbient   = float4(0.0f, 0.0f, 0.0f, 0.0f);
float  GlowThickness = 5.025f;

struct VSTEXTURE_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse  : COLOR;
    float2 TexCoord : TEXCOORD0;
};

// draws unskinned object with one texture and one directional light.
VSTEXTURE_OUTPUT VSTexture
    (
    float4 Position : POSITION, 
    float3 Normal   : NORMAL,
    float2 TexCoord : TEXCOORD0
    )
{
    VSTEXTURE_OUTPUT Out = (VSTEXTURE_OUTPUT)0;
  
    float3 L = -LightDir;                                   // light direction (view space)
    float3 P = mul(Position, WorldView);                    // position (view space)
    float3 N = normalize(mul(Normal, (float3x3)WorldView)); // normal (view space)
   
    Out.Position = mul(float4(P, 1), Projection);   // projected position
    Out.Diffuse  = max(0, dot(N, L));               // diffuse 
    Out.TexCoord = TexCoord;                        // texture coordinates

/*
    float4 rainbow = float4(
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 0)),
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 2*3.14159/3)),
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 4*3.14159/3)),
                            1.0
                        );    
    float4 rainbow2 = float4(0.0,-16.0,175.5,1.0) + rainbow;

Out.Diffuse = rainbow2;
*/

//Out.Diffuse = GlowColor;
Out.Diffuse = float4(1,1,1,1);

//Out.Diffuse = WorldView  ;

    return Out;    
}

struct VSGLOW_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse  : COLOR;
    float2 TexCoord : TEXCOORD0;
};

// draws a transparent hull of the unskinned object.
VSGLOW_OUTPUT VSGlow
    (
    float4 Position : POSITION, 
    float3 Normal   : NORMAL,
    float2 TexCoord : TEXCOORD0
    )
{
    VSGLOW_OUTPUT Out = (VSGLOW_OUTPUT)0;

    Out.TexCoord = TexCoord;                        // texture coordinates

    float3 N = normalize(mul(Normal, (float3x3)WorldView));     // normal (view space)
    float3 P = mul(Position, WorldView) + GlowThickness * N;    // displaced position (view space)

    Out.Position = mul(float4(P, 1), Projection);   // projected position
    Out.Diffuse  = GlowColor; // modulated glow color + glow ambient

Out.Diffuse.x = 1;
Out.Diffuse.y = 1;
Out.Diffuse.z = 1;

//Out.Diffuse = float4(1.75,1.75,1.75,1);

    return Out;    
}



technique TGlowAndTexture
{
    pass PTexture
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSTexture();
        PixelShader  = NULL;
        
        // texture
        Texture[0] = (Tex0);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        AlphaOp[0]   = DISABLE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;
        
VertexShaderConstant[7] = float4(
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 0)),
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 2*3.14159/3))-0.0625,
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 4*3.14159/3))+0.685546875,
                            1.0
                        );
    }

    pass PGlow
    {   
        // glow shader
        VertexShader = compile vs_1_1 VSGlow();
        PixelShader  = NULL;
        
        // texture
        Texture[0] = (Tex0);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // enable alpha blending
        AlphaBlendEnable = TRUE;
        DitherEnable = TRUE;
        SrcBlend         = ONE;
        DestBlend        = ONE;
        BlendOp = REVSUBTRACT;
//        SrcBlend         = ZERO;
//        DestBlend        = INVSRCCOLOR;
//        BlendOp = ADD;

        // set up texture stage states to use the diffuse color
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = SELECTARG2;
        AlphaArg2[0] = DIFFUSE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

VertexShaderConstant[7] = float4(
                            1-(GetOscillatingValue(float4(0.0, 1.0, 1.5, 0))/2),
                            1-(GetOscillatingValue(float4(0.0, 1.0, 1.5, 2*3.14159/3))/2-0.0625),
                            1-(GetOscillatingValue(float4(0.0, 1.0, 1.5, 4*3.14159/3))/2+0.685546875),
                            1.0
                        );
/*
VertexShaderConstant[7] = float4(
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 0)),
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 2*3.14159/3)),
                            GetOscillatingValue(float4(0.0, 1.0, 1.5, 4*3.14159/3)),
                            1.0
                        );
*/


//    float4 rainbow2 = float4(0.0,-16.0,175.5,1.0) + rainbow;

//    float4 rainbow3 = 1-(saturate(rainbow2) - saturate(rainbow/2));

    }
}

