//
// Wood Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//

//string XFile = "misc\\teapot.x";
string XFile = "teapot.x";
int    BCLR  = 0xff202080;

// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;

// light direction (view space)
half3 lightDir <  string UIDirectional = "Light Direction"; > = {0.577, -0.577, 0.577};

// light intensity
half4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
half4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
half4 I_s = { 0.6f, 0.6f, 0.6f, 1.0f };    // specular

// material reflectivity
half4 k_a = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient
half4 k_d = { 1.0f, 0.7f, 0.2f, 1.0f };    // diffuse
half4 k_s = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular
half  n   = 64.0f;                         // power

half3 k_d_negative = { 0.35f, 0.4f, 1.0f }; // an inverse amount of the diffuse, for vs-only specular


#define DARK    0.3f
#define LIGHT   1.0f

// model dependent wood parameters
#define RINGSCALE   10.0f
#define POINTSCALE  2.0f
#define TURBULENCE  1.0f
#define VOLUME_SIZE 32

half time : TIME;

// textures

texture LinearTex < string function = "Linear"; int width = 16; int height = 16; >;
texture NoiseTex  < string type = "VOLUME"; string function = "GenerateNoise"; int width = VOLUME_SIZE, height = VOLUME_SIZE, depth = VOLUME_SIZE; >;

// samplers
sampler NoiseSamp = sampler_state 
{
    texture = <NoiseTex>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

sampler LinearSamp = sampler_state 
{
    texture = <LinearTex>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture NoiseTexCube
< 
//    string type = "CUBE"; 
    string function = "GenerateNoiseCube"; 
    int width = 128;
    int height = 128;
>;


half4 Linear(half2 Pos : POSITION) : COLOR
{
    return half4(Pos, Pos);
}

// function used to fill the volume noise texture
half4 GenerateNoise(half3 Pos : POSITION) : COLOR
{
    half4 Noise = (half4)0;

    for(int i = 1; i < 256; i += i)
    {
        Noise.r += abs(noise(Pos * 500 * i)) / i;
        Noise.g += abs(noise((Pos + 1)* 500 * i)) / i;
        Noise.b += abs(noise((Pos + 2) * 500 * i)) / i;
        Noise.a += abs(noise((Pos + 3) * 500 * i)) / i;
    }

    return Noise;
}

// function used to fill the volume noise texture
half4 GenerateNoiseCube(half2 Pos : POSITION) : COLOR
{
 /*   half4 Color;
    half3 tex1 = 0.5 * Pos * POINTSCALE;

    half ringdist = (Pos.x*Pos.x + Pos.z*Pos.z);

    half r = 4*Pos.y - ringdist + .75 + .5*(noise(tex1*14.3)-.5);

    if(Pos.y == -1 && ringdist < 1)
    r -= 5*ringdist;

    if(Pos.y == 1 && ringdist < 1)
    r += 5*ringdist;

    r = frac(length(r));

    Color.rgb = lerp(DARK, LIGHT, saturate(0.8 - 0.6 * r));
    Color.rgb *= I_a * k_a + I_d * k_d;
    Color.w   = 1;

    return Color;*/
    
//    half3 tex1 = 0.5 * Pos * POINTSCALE;
    return noise(85*Pos.xy);
}

struct VS_OUTPUT
{
    half4 Pos  : POSITION;
    half3 Diff : COLOR0;
    half3 Spec : COLOR1;
    half3 Tex0 : TEXCOORD0;               
    half3 Tex1 : TEXCOORD1;               
    half2 Tex2 : TEXCOORD2;               
};

VS_OUTPUT VS(    
    half3 Pos  : POSITION,
    half3 Norm : NORMAL)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    half3 L = -lightDir;
    half3 P = mul(half4(Pos, 1), (half4x3)WorldView);    // position (view space)
    half3 N = normalize(mul(Norm, (half3x3)WorldView));   // normal (view space)
    half3 R = normalize(2 * dot(N, L) * N - L);            // reflection vector (view space)
    half3 V = -normalize(P);                               // view direction (view space)

    Out.Pos  = mul(half4(P, 1), Projection);             // position (projected)
    Out.Diff = I_a * k_a + I_d * k_d * max(0, dot(N, L)); // diffuse + ambient
    Out.Spec = I_s * k_s * pow(max(0, dot(R, V)), n/4);   // specular
    Out.Tex1 = 0.5 * Pos * POINTSCALE; 
    Out.Tex0 = Out.Tex1 * TURBULENCE;        
    Out.Tex2 = RINGSCALE * length(Out.Tex1.xz);

    return Out;
}


struct VSforFF_OUTPUT
{
    half4 Pos  : POSITION;
    half3 Diff : COLOR0;
    half2 Tex0 : TEXCOORD0;               
};

VSforFF_OUTPUT VSforFF(    
    half3 Pos  : POSITION,
    half3 Norm : NORMAL)
{
    VSforFF_OUTPUT Out = (VSforFF_OUTPUT)0;

    half3 L = -lightDir;
    half3 P = mul(half4(Pos, 1), (half4x3)WorldView);    // position (view space)
    half3 N = normalize(mul(Norm, (half3x3)WorldView));   // normal (view space)
    half3 R = normalize(2 * dot(N, L) * N - L);            // reflection vector (view space)
    half3 V = -normalize(P);                               // view direction (view space)

    Out.Pos  = mul(half4(P, 1), Projection);             // position (projected)
    Out.Diff = 0.5*max(0, dot(N, L))  // diffuse + ambient
             + 2.0*I_s * (min(k_s,k_d_negative)) * pow(max(0, dot(R, V)), n/4);   // specular
    Out.Tex0 = 0.5 * sin(Pos*2) * POINTSCALE + 0.1*(sin(Pos.x)+sin(Pos.y)+sin(Pos.z)) * POINTSCALE ;
    return Out;
}




// PS11 version is a gross approximation
half4 PS11(VS_OUTPUT In) : COLOR
{   
    half4 Color;
    half r;

    // note the use of a linear texture with wrapped texcoords to emulate 'frac'
    r = tex2D(LinearSamp, In.Tex2) +  0.1 * (tex3D(NoiseSamp, In.Tex1) - 0.5); 

    Color.rgb = In.Diff * lerp(DARK, LIGHT, saturate(0.8 - 0.6 * r))
              + In.Spec;
    Color.w   = 1;

    return Color;
}  

// PS20 is much better than PS11 because of dependent texture reads
half4 PS20(VS_OUTPUT In) : COLOR
{   
    half4 Color;
    half3 PP = In.Tex1 + 0.1 * (tex3D(NoiseSamp, frac(In.Tex1)) - 0.5);
    half r;

    r = RINGSCALE * length(PP.xz);
    r += 0.1 * tex3D(NoiseSamp, r);
    r = frac(r);

    Color.rgb = In.Diff * lerp(DARK, LIGHT, saturate(0.8 - 0.6 * r))
              + In.Spec;
    Color.w   = 1;

    return Color;
}  


technique TWood_VS_1_1
{
    pass P0
    {
        VertexShader = compile vs_1_1 VSforFF();
        PixelShader = NULL;

        DitherEnable = TRUE;
        AlphaBlendEnable = FALSE;
        CullMode         = CCW;

Lighting = false;
SpecularEnable = false;

        AlphaBlendEnable = FALSE;
        CullMode         = CCW;
        DitherEnable = TRUE;

        // texture
        Texture[0] = (NoiseTexCube);
        Texture[1] = (NoiseTexCube);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = NONE;

        MinFilter[1] = LINEAR;
        MagFilter[1] = LINEAR;
        MipFilter[1] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
//        ColorOp[0]   = ADDSIGNED2X;
        ColorOp[0]   = DOTPRODUCT3;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;

        ColorOp[1]   = MODULATE;
//        ColorOp[1]   = DISABLE;
        ColorArg1[1] = DIFFUSE;
        ColorArg2[1] = CURRENT;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = FALSE;
        SrcBlend         = ONE;
        DestBlend        = ZERO;
//        BlendOp = SUBTRACT;
    }
}


technique TWood_PS_1_1
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_1 PS11();
    }
}

technique TWood_PS_2_0
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS20();
    }
}
