//
// Glow Effect
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//
// newest working


float4x3 WorldView  : WORLDVIEW; 
float4x4 Projection : PROJECTION;
float4x4 WorldViewProj  : WORLDVIEWPROJECTION; 
float4x4 _world : WORLD; 
float4x4 View  : VIEW; 
float MeshRadius : MESHRADIUS;

float3 LightPos = float3(0,0,0);
float4 LightColor = float4(1,1.5,2,1);
float4 AmbientFull = float4(0,0,0,0);
float LightStrength = 1; // = 1/attenuation1

float time : TIME;


//float3 LightPos = (float3(0.0f, 0.0f, 100.0f));
//float3 LightPos = (float3(50.0f, -50.0f, 75.0f));
//float  LightRange = 175.0f;
//float4 LightDiffuseAmbient = float4(1.0,0.5,0.0,0);
//float4 AmbientFull = float4(0.15,0.15,0.25,0);



string XFile = "skullocc.x";   // model
string BIMG  = "lake.bmp";  // Background image
int    BCLR  = 0xff202080;  // background

// texture
texture Tex0 < string name = "tiny_skin.bmp"; >;
texture TexE < string name = "lake.bmp"; >;

// glow parameters
float4 GlowEdgeColor1 = float4(-1,-1,-1,0);
float4 GlowEdgeColor2 = float4(1.0-0.0f, 0.4-0.4f, 0.0-0.8f, 1.0f);

float4 GlowAmbient1   = float4(0.0f, 0.0f, 0.0f, 0.0f);
float4 GlowAmbient2   = float4(0.0f, 0.0f, 0.0f, 0.0f);

float4 GlowColor1     = float4(    0.0f,     0.4f,     0.8f, 1.0f);
float4 GlowColor2     = float4(    0.5f,     0.25f,    0.6f, 1.0f);

//float  GlowThickness = 3.025f;
//float  GlowThickness2 = 6.025f;


struct VSGLOW_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse  : COLOR;
//    float2 TexCoord : TEXCOORD;
};

// draws a transparent hull of the unskinned object.
VSGLOW_OUTPUT VSGlow
    (
    float4 Position : POSITION, 
    float3 Normal   : NORMAL,
    uniform float4 GlowAmbient,
    uniform float4 GlowColor,
    uniform float4 GlowEdgeColor,
    uniform float GlowThickness
    )
{
    VSGLOW_OUTPUT Out = (VSGLOW_OUTPUT)0;

    float3 N = normalize(mul(Normal, (float3x3)WorldView));     // normal (view space)
    float3 P = mul(Position, WorldView) + GlowThickness;    // displaced position (view space)
    float3 A = float3(0, 0, 1);                                 // glow axis

    float Power  = dot(N, A);
//    Power *= Power;
//    Power -= 1;
//    Power *= Power;     // Power = (1 - (N.A)^2)^2 [ = ((N.A)^2 - 1)^2 ]
    
    Out.Position = mul(float4(P, 1), Projection);   // projected position
//    Out.Diffuse = GlowColor + GlowAmbient; // Inner Blue layer
//    Out.Diffuse += GlowEdgeColor * Power; // Outer Red layer
//    Out.TexCoord.x = cos(N.x*8) + time*0; // animate the blue layer
  //  Out.TexCoord.y = sin(N.y*8) + time*0;


    Out.Diffuse = float4(1,0,0,0);
if(Power > -.4)
{
//    Power = 0;
    Out.Diffuse = float4(0,0,0,0);
}
if(Power < -.6)
{
//    Power = -1;
    Out.Diffuse = float4(1,1,1,1);
}

//Out.Diffuse += GlowEdgeColor * Power;

//Out.Diffuse = normalize(float4(-Power,0,0,.9));

//Out.Diffuse = normalize(Out.Diffuse);

    return Out;    
}






struct VSTEXTURE_OUTPUT
{
    float4 Position : POSITION;
    float4 Diffuse  : COLOR;
    float2 TexCoord : TEXCOORD0;
};

// draws unskinned object with one texture and one directional light.
VSTEXTURE_OUTPUT VSTexture
    (
    float4 Position : POSITION, 
    float3 Normal   : NORMAL,
    float2 TexCoord : TEXCOORD0
    )
{
    VSTEXTURE_OUTPUT Out = (VSTEXTURE_OUTPUT)0;
  
    Out.Position = mul(Position, WorldViewProj);          // position (view space)
    float3 N = mul(Normal, (float3x3)WorldView);  // normal (view space)

    float3 PWorld = mul(Position, _world);          // position (world space)

    // float3 LightPos = (-normalize(float3(0.0f, 0.0f, 1.0f)) * 100);

    Out.TexCoord = TexCoord;                            // texture coordinates

    float3 LToV = PWorld - LightPos;
    float dist = (half)length(LToV);
    float3 LDir = LToV/dist;

    Out.Diffuse = max(0, -(half)dot(N, LDir)); // calculate lighting from orientation 
    Out.Diffuse *= LightColor;
//    Out.Diffuse += LightDiffuseAmbient;
    Out.Diffuse *= LightStrength/dist; // calculate lighting from distance
    Out.Diffuse += AmbientFull; // Add ambient lighting

    return Out;    
}

struct VSOUTLINE_OUTPUT
{
    float4 Position : POSITION;
//    float4 Diffuse  : COLOR;
};

// draws an outline around the object
VSOUTLINE_OUTPUT VSOutline
    (
    float4 Position : POSITION, 
    float3 Normal   : NORMAL,
    uniform float GlowThickness
    )
{
    VSOUTLINE_OUTPUT Out = (VSOUTLINE_OUTPUT)0;
  
    float3 N = normalize(mul(Normal, (float3x3)WorldView)); // normal (view space)
    float3 P = mul(Position, WorldView) + GlowThickness * N;// position (view space)
   
    Out.Position = mul(float4(P, 1), Projection);   // projected position

    return Out;    
}


technique TOutlineAndTextureWithVertexShader
{
/*    pass PTexture
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSTexture();
//        VertexShader = compile vs_1_1 VSGlow(GlowAmbient2,GlowColor2/2,GlowColor1*2+float4(sin(time),0,0,0),MeshRadius/32*sin(time));
        PixelShader  = NULL;
        
        // texture
        Texture[0] = (Tex0);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = FALSE;
        SrcBlend = ONE;
        DestBlend = ZERO;
        DitherEnable = TRUE;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = TRUE;
    }*/
    pass PGlowLayer1
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSGlow(GlowAmbient1,GlowColor1,GlowEdgeColor1,MeshRadius/128);
        PixelShader  = NULL;
        
//        VertexShaderConstant[11] = (MeshRadius/128);

        // texture
        Texture[0] = (TexE);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        DitherEnable = TRUE;
        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
//        DestBlend = ONE;
        DestBlend = ZERO;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = TRUE;
     }
    pass PGlowLayer2
    {   
        // single texture/one directional light shader
        VertexShader = compile vs_1_1 VSGlow(GlowAmbient2,GlowColor2,GlowEdgeColor2,MeshRadius/64);
        PixelShader  = NULL;
        
//        VertexShaderConstant[11] = (MeshRadius/64);

        // texture
        Texture[0] = (TexE);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
        DestBlend = ONE;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;
     }
}



// light intensity
float4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse

// material reflectivity
float4 k_a : MATERIALAMBIENT = { 1.0f, 1.0f, 1.0f, 1.0f };    // ambient
float4 k_d : MATERIALDIFFUSE = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
//float4 k_s : MATERIALSPECULAR= { 1.0f, 1.0f, 1.0f, 1.0f };    // specular
//int    n   : MATERIALPOWER = 32;                            // power

float g = 1.075;

technique TOutlineAndTextureNoShader
{
    pass TEXTUREPASS
    {

        // transforms
        WorldTransform[0]   = (_world);
        ViewTransform       = (View);
        ProjectionTransform = (Projection);

        // material
        MaterialAmbient  = (float4(1,1,1,0)); 
        MaterialDiffuse  = (k_d); 
        MaterialSpecular = (0); 
        MaterialPower    = (0);
        
        // lighting
        LightType[0]      = POINT;
//        LightAmbient[0]   = (LightDiffuseAmbient);
        LightAmbient[0]   = (0);
        Ambient = (AmbientFull);
        LightDiffuse[0]   = (LightColor);
        LightSpecular[0]  = (0); 

        LightPosition[0] = (LightPos);
        LightRange[0]     = 100000.0f;

        LightAttenuation0[0] = (0.0);
        LightAttenuation1[0] = (1/LightStrength);
        LightAttenuation2[0] = (0.0);


        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = FALSE;
        
        // texture
        Texture[0] = (Tex0);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;
        
        // texture stages
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = DIFFUSE;
        AlphaOp[0]   = MODULATE;
        AlphaArg1[0] = TEXTURE;
        AlphaArg2[0] = DIFFUSE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = FALSE;
        SrcBlend = ONE;
        DestBlend = ZERO;
        DitherEnable = TRUE;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = TRUE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }

    pass BLUELAYER
    {
        // transforms
        WorldTransform[0]   = (_world + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,4*(MeshRadius/64),0));
        ViewTransform       = (View);
        ProjectionTransform = (Projection + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,-.5,0));

        Lighting       = TRUE;
        Ambient = (GlowColor1/2 + GlowColor2/3 * (2+sin(time*2+.78))/2 + GlowAmbient1/4);
        MaterialAmbient  = (float4(1,1,1,0)); 
        MaterialDiffuse  = (0); 
        MaterialSpecular = (0); 
        MaterialPower    = (0);
        LightRange[0]    = (0);
       
        // no texture
        Texture[0] = <TexE>;
        TexCoordIndex[0] = CAMERASPACENORMAL;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0]  = TEXTURE;
        ColorArg2[0]  = DIFFUSE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
        DestBlend = ONE;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;

    }

    pass REDOUTLINEONE
    {
        // transforms
        WorldTransform[0]   = (_world);
        ViewTransform       = (View);
        ProjectionTransform = (Projection + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, 2*(MeshRadius/64),2*(MeshRadius/64),.005*(MeshRadius/64),0));

        Lighting       = TRUE;
        Ambient = (GlowEdgeColor1*1/3 * (2+cos(time*2))/2 + GlowColor1*2/6 * (2+sin(time*2))/2+ GlowAmbient1/3);
        MaterialAmbient  = (float4(1,1,1,0)); 
        MaterialDiffuse  = (0); 
        MaterialSpecular = (0); 
        MaterialPower    = (0);
        LightRange[0]    = (0);
       
        // no texture
        Texture[0] = NULL;
        TexCoordIndex[0] = PASSTHRU;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = SELECTARG1;
        ColorArg1[0]  = DIFFUSE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
        DestBlend = ONE;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;

    }
    pass REDOUTLINETWO
    {
        // transforms
        WorldTransform[0]   = (_world);
        ViewTransform       = (View);
        ProjectionTransform = (Projection + matrix(0,0,0,0, 0,0,0,0, 0,0,0,0, -(MeshRadius/64),-(MeshRadius/64),.0167*(MeshRadius/64),0));

        Lighting       = TRUE;
        Ambient = (GlowEdgeColor2*1/3 * (2+sin(time*2))/2 + GlowAmbient2/3);
        MaterialAmbient  = (float4(1,1,1,0)); 
        MaterialDiffuse  = (0); 
        MaterialSpecular = (0); 
        MaterialPower    = (0);
        LightRange[0]    = (0);
       
        // no texture
        Texture[0] = NULL;
        TexCoordIndex[0] = PASSTHRU;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = SELECTARG1;
        ColorArg1[0]  = DIFFUSE;
        AlphaOp[0]   = DISABLE;
        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend = ONE;
        DestBlend = ONE;

        CullMode = CCW;
        ZEnable = TRUE;
        ZWriteEnable = FALSE;

        // shaders
        VertexShader = NULL;
        PixelShader  = NULL;

    }
}
