//
// Water Shader
//
// Note: This effect file works with EffectEdit.


// sparkle parameters
#define SPRINKLE    0.3
#define SCATTER     0.3

#define VOLUME_NOISE_SCALE  10

// scene info (EffectEdit specific controls)
string XFile = "plane.x";
int    BCLR  = 0xff444444;
string BIMG  = "misc\\lake.bmp";  // Background image

// textures

//texture TinyMap < string name = "Data\\Images\\Effects\\env3.bmp" ; >;
// textures

texture EnvironmentMap 
< 
    string type = "CUBE"; 
    string name = "Data\\Images\\Textures\\lobbycube.dds"; 
>;

//texture EnvironmentMap < string name = "Data\\Images\\Effects\\env3.bmp" ; >;
//texture EnvironmentMap < string name = "Data\\Images\\Textures\\blue.dds" ; >;


// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;
half4x4 WorldViewProj : WORLDVIEWPROJECTION;

// light direction (view space)
half3 L < string UIDirectional = "Light Direction"; > = normalize(half3(-0.397f, -0.397f, 0.827f));

// material
half4 k_a = { 0.0f, 0.0f, 0.5f, 0.5f };    // ambient  (metal)
half4 k_d = { 0.6f, 0.8f, 1.0f, 0.5f };    // diffuse  (metal)
half4 k_s = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular (metal)

half time : TIME;

half amplitude = 2;
half wavefreq = 0.125;
//half camfreq = 0.05;

//half3 campos : CAMERAPOSITION;

struct VS_OUTPUT
{
    half4 Position   : POSITION;
    half4 Diffuse    : COLOR0;
    half3 TexCoord   : TEXCOORD0;
};

// vertex shader
VS_OUTPUT VS (half3 Position : POSITION)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

	half xwave = time + Position.x*wavefreq; // wave angle
	half ywave = time + Position.y*wavefreq;
    half xzplus = sin(xwave); // wave height
    half yzplus = sin(ywave);
//    half xnorm = cos(xwave+campos.x*camfreq*0); // normal (derivative of height)
//    half ynorm = cos(ywave+campos.y*camfreq*0);
    half xnorm = cos(xwave); // normal (derivative of height)
    half ynorm = cos(ywave);

	half amplitudeModulator = (Position.x*Position.x + Position.y*Position.y) * (1.0f/(64*64));
	
	// add wave height to position
    Position.z += (xzplus + yzplus - 1.5) * amplitude * amplitudeModulator;
	
	// project the position
//	Out.Position = mul(float4(Position,1), WorldViewProj);
    half3 P = mul(half4(Position, 1), (half4x3)WorldView);   // position (view space)
    Out.Position = mul(half4(P, 1), Projection);             

	// set normal to (some gross approximation of the) wave normal and project normal to view space
    half3 Normal = half3(xnorm,ynorm,(xzplus*yzplus));
    half3 N = normalize(mul(Normal,(half3x3)WorldView));
	half3 V = normalize(P);

	// calculate shading -- ambient + diffuse
    Out.Diffuse = k_a + (k_d) * max(0, dot(N, V));


    // environment cube map coordinates
    half3 G = normalize(-2 * dot(N, V) * N + V);                // glance vector (view space)
    Out.TexCoord = half3(-G.x, G.y, -G.z);


    return Out;
}



// technique
technique TWater
{
   pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader = NULL;

         // texture
        Texture[0] = (EnvironmentMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

       // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = DIFFUSE;
        ColorArg2[0] = TEXTURE;
        AlphaOp[0]   = SELECTARG1;
        AlphaArg1[0] = DIFFUSE;
        AlphaArg2[0] = TEXTURE;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

        AlphaBlendEnable = TRUE;
        SrcBlend         = ONE; // this should be SRCALPHA but this is a little faster and enhances brightness
        DestBlend        = INVSRCALPHA;
        SpecularEnable = FALSE;

        CullMode         = CCW;
        ZWriteEnable     = FALSE; // should change this to TRUE if the water is drawn last for less-weird wave overlaps
        DitherEnable = TRUE;

    }
}

